#ifndef PROCESSWIDGET_H
#define PROCESSWIDGET_H

#include <QWidget>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGSettings>
#include <QSettings>
#include <QButtonGroup>

#include "../controls/kgroupbutton.h"
#include "processtableview.h"

class ProcessWidget : public QWidget
{
    Q_OBJECT
public:
    enum {
      PROCESSTYPE_ACTIVE,
      PROCESSTYPE_BELONGME,
      PROCESSTYPE_ALL,
      PROCESSTYPE_APPLICATION
    };
    explicit ProcessWidget(QSettings *proSettings = nullptr, QWidget *parent = nullptr);
    virtual ~ProcessWidget();

    virtual void onWndClose();

public slots:
    void onSwitchProcType(int nType);

signals:
    void changeProcessItemDialog(int itemIndex);
    void searchSignal(QString);

protected:
    void paintEvent(QPaintEvent *) override;

private:
    void initUI();
    void initConnections();
    void initThemeMode();
    void onThemeFontChanged(float fFontSize);

private:
    QVBoxLayout *m_mainLayout = nullptr;
    QHBoxLayout *m_topBtnsLayout = nullptr;
    QVBoxLayout *m_procTableLayout = nullptr;

    KGroupButton *m_btnApplication = nullptr;
    KGroupButton *m_btnActive = nullptr;
    KGroupButton *m_btnBelongMe = nullptr;
    KGroupButton *m_btnAll = nullptr;
    QButtonGroup *m_buttonGroup = nullptr;
    int m_nWhichNum = 0;

    ProcessTableView *m_procTabView = nullptr;
    QGSettings *m_qtSettings = nullptr;
    QGSettings *m_ifSettings = nullptr;
    QSettings *m_proSettings = nullptr;
};

#endif // PROCESSWIDGET_H
