#include "processwndinfo.h"

#include <QtX11Extras/QX11Info>

ProcessWndInfo::ProcessWndInfo(QObject* parent)
    : QObject(parent)
{

}

ProcessWndInfo::~ProcessWndInfo()
{

}

void ProcessWndInfo::updateWindowInfos()
{
    const auto wnds = KWindowSystem::stackingOrder();
    m_mapProcWndInfos.clear();
    for (auto const wnd: wnds) {
        if (acceptWindow(wnd)) {
            ProcWindowInfo wndInfo;
            wndInfo.wndId = wnd;
            KWindowInfo kwndInfo(wnd, NET::WMName|NET::WMVisibleName|NET::WMIcon);
            if (kwndInfo.valid()) {
                wndInfo.procId = kwndInfo.pid();
                wndInfo.wndName = kwndInfo.visibleName().isEmpty() ? kwndInfo.name() : kwndInfo.visibleName();
                wndInfo.wndIcon = KWindowSystem::icon(wnd);
                m_mapProcWndInfos[wndInfo.procId] = wndInfo;
            }
        }
    }
}

bool ProcessWndInfo::acceptWindow(WId window)
{
    QFlags<NET::WindowTypeMask> ignoreList;
    ignoreList |= NET::DesktopMask;
    ignoreList |= NET::DockMask;
    ignoreList |= NET::SplashMask;
    ignoreList |= NET::ToolbarMask;
    ignoreList |= NET::MenuMask;
    ignoreList |= NET::PopupMenuMask;
    ignoreList |= NET::NotificationMask;

    KWindowInfo info(window, NET::WMWindowType | NET::WMState, NET::WM2TransientFor);
    if (!info.valid())
        return false;

    if (NET::typeMatchesMask(info.windowType(NET::AllTypesMask), ignoreList))
        return false;

    if (info.state() & NET::SkipTaskbar)
        return false;

    // WM_TRANSIENT_FOR hint not set - normal window
    WId transFor = info.transientFor();
    if (transFor == 0 || transFor == window || transFor == (WId) QX11Info::appRootWindow())
        return true;

    info = KWindowInfo(transFor, NET::WMWindowType);

    QFlags<NET::WindowTypeMask> normalFlag;
    normalFlag |= NET::NormalMask;
    normalFlag |= NET::DialogMask;
    normalFlag |= NET::UtilityMask;

    return !NET::typeMatchesMask(info.windowType(NET::AllTypesMask), normalFlag);
}

QList<pid_t> ProcessWndInfo::getWindowPids()
{
    return m_mapProcWndInfos.keys();
}

WId ProcessWndInfo::getWndIdByPid(pid_t pid)
{
    if (m_mapProcWndInfos.contains(pid)) {
        return m_mapProcWndInfos[pid].wndId;
    }
    return 0;
}

QString ProcessWndInfo::getWndNameByPid(pid_t pid)
{
    if (m_mapProcWndInfos.contains(pid)) {
        return m_mapProcWndInfos[pid].wndName;
    }
    return "";
}

QIcon ProcessWndInfo::getWndIconByPid(pid_t pid)
{
    if (m_mapProcWndInfos.contains(pid)) {
        return m_mapProcWndInfos[pid].wndIcon;
    }
    return QIcon();
}
